Program Fractals;


(**************************************************************************)
(*                                                                        *)
(*  Fractals 1.0, a demo program for GNU Pascal and its libraries which   *)
(*  graphically displays the following fractal sets:                      *)
(*    1) Julia set of the polynomial x**3 - (1+i),                        *)
(*    2) the Mandelbrot set.                                              *)
(*                                                                        *)
(*  Copyright (C) 1997  Peter Gerwinski, Essen, Germany                   *)
(*                      peter.gerwinski@uni-essen.de                      *)
(*                      http://home.pages.de/~peter.gerwinski/            *)
(*                                                                        *)
(*  This program is free software; you can redistribute it and/or modify  *)
(*  it under the terms of the GNU General Public License, version 2, as   *)
(*  published by the Free Software Foundation.  You can use parts of      *)
(*  this program in your own programs under the terms of the GNU Library  *)
(*  General Public License, version 2.                                    *)
(*                                                                        *)
(*  This program is distributed in the hope that it will be useful,       *)
(*  but WITHOUT ANY WARRANTY; without even the implied warranty of        *)
(*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *)
(*  GNU General Public License for more details.                          *)
(*                                                                        *)
(*  You should have received a copy of the GNU General Public License     *)
(*  and the GNU Library General Public License along with this program;   *)
(*  if not, write to the Free Software Foundation, Inc., 675 Mass Ave,    *)
(*  Cambridge, MA 02139, USA.                                             *)
(*                                                                        *)
(**************************************************************************)


(* Fr Anja. *)


(*$X+*)


uses
  System, Crt, Graph;


Type
  ColorFunc = ^Function ( Real, Real ): Integer;


Var
  GraphDriver: Integer = native_GRX;
  GraphMode: Integer = GRX_biggest_graphics;


Const
  GraphPath = 'doesn''t matter';


Const
  epsilon = 1E-15;


Var
  x0: Real value - 0.5;
  y0: Real value 0.0;
  size: Real value 2.0;
  Choice, Ch: Char;


(* The function for the Julia set *)

Function F ( x: Complex ): Complex;

begin (* F *)
  F:= x pow 3 - cmplx ( 1, 1 );
end (* F *);


(* Its derivative *)

Function dF ( x: Complex ): Complex;

begin (* dF *)
  dF:= cmplx ( 3, 0 ) * sqr ( x );
end (* dF *);


(* Newton's algorithm for finding zeros *)

Function Newton ( z: Complex ): Complex;

Var
  Fz, dFz: Complex;

begin (* Newton *)
  repeat
    Fz:= F ( z );
    dFz:= dF ( z );
    if abs ( dFz ) >= epsilon then
      z:= z - Fz / dFz;
  until ( abs ( Fz ) < epsilon )
     or ( abs ( dFz ) < epsilon )
     or ( abs ( z ) > 1E30 );
  Newton:= z;
end (* Newton *);


(* Draw a solid square *)

Procedure Square ( x, y, a, color: Integer );

begin (* Square *)
  SetFillStyle ( solidfill, color );
  Bar ( x, y, x + a - 1, y + a - 1 );
end (* Square *);


(* Draw the graph with increaing resolution *)

Procedure ColorGraph ( x0, x1, y0, y1: Real; Color: ColorFunc );

Var
  ix, iy, Step, c: Integer;
  x, y: Real;
  complete: Boolean;

begin (* ColorGraph *)
  Step:= 64;
  iy:= 0;
  while ( iy <= GetMaxY ) and not KeyPressed do
    begin
      y:= y1 + ( y0 - y1 ) * ( iy / GetMaxY );
      ix:= 0;
      while ( ix <= GetMaxX ) and not KeyPressed do
        begin
          x:= x0 + ( x1 - x0 ) * ( ix / GetMaxX );
          Square ( ix, iy, Step, white );
          c:= Color^ ( x, y );
          Square ( ix, iy, Step, c );
          inc ( ix, Step );
        end (* while *);
      inc ( iy, Step );
    end (* while *);
  Step:= Step div 2;
  while ( Step > 0 ) and not KeyPressed do
    begin
      iy:= 0;
      complete:= false;
      while ( iy <= GetMaxY ) and not KeyPressed do
        begin
          y:= y1 + ( y0 - y1 ) * ( iy / GetMaxY );
          if complete then
            ix:= 0
          else
            ix:= Step;
          while ( ix <= GetMaxX ) and not KeyPressed do
            begin
              x:= x0 + ( x1 - x0 ) * ( ix / GetMaxX );
              Square ( ix, iy, Step, white );
              c:= Color^ ( x, y );
              Square ( ix, iy, Step, c );
              if complete then
                inc ( ix, Step )
              else
                inc ( ix, 2 * Step );
            end (* while *);
          inc ( iy, Step );
          complete:= not complete;
        end (* while *);
      Step:= Step shr 1;
    end (* while *);
end (* ColorGraph *);


(* The Graph of the Julia set *)

Function JuliaColor ( x, y: Real ): Integer;

Var
  zero: Complex;

begin (* JuliaColor *)
  zero:= Newton ( cmplx ( x, y ) );
  if Im ( zero ) > 0.5 then
    JuliaColor:= green
  else if Im ( zero ) > 0 then
    JuliaColor:= yellow
  else
    JuliaColor:= red;
end (* JuliaColor *);


(* The Graph of the Mandelbrot set *)

Function MandelColor ( xx, yy: Real ): Integer;

Const
  tmax = 999;

Var
  t: Integer;
  x, y, z: Real;

begin (* MandelColor *)
  MandelColor:= black;
  t:= 0;
  if ( sqr ( xx + 0.24 ) + sqr ( yy ) > 0.24 )
   and ( sqr ( xx + 1 ) + sqr ( yy ) > 0.05 ) then
    begin
      x:= 0;
      y:= 0;
      repeat
        z:= sqr ( x ) - sqr ( y ) + xx;
        y:= 2 * x * y + yy;
        x:= z;
        inc ( t );
      until ( abs ( z ) > 100 ) or ( t > tmax );
      if t <= tmax then
        MandelColor:= t mod GetMaxColor + 1;
    end (* if *);
end (* MandelColor *);


begin
  ClrScr;
  writeln ( 'arrows  -  walk' );
  writeln ( '+ -     -  zoom' );
  writeln ( 'ESC     -  quit' );
  writeln;
  writeln ( '1       -  Julia set of the polynomial x**3 - (1+i)' );
  writeln ( '2       -  Mandelbrot set' );
  repeat
    Choice:= ReadKey;
  until Choice in [ '1', '2', chr ( 27 ) ];
  if Choice <> chr ( 27 ) then
    begin
      InitGraph ( GraphDriver, GraphMode, GraphPath );
      repeat
        (* It would be better to use a variable of *)
        (* type `ColorFunc' here, but due to a bug *)
        (* in GPC that does not work. :-(          *)
        case Choice of
          '1': ColorGraph ( x0 - 0.8 * size, x0 + 0.8 * size,
                            y0 - 0.6 * size, y0 + 0.6 * size, @JuliaColor );
          '2': ColorGraph ( x0 - 0.8 * size, x0 + 0.8 * size,
                            y0 - 0.6 * size, y0 + 0.6 * size, @MandelColor );
        end (* case *);
        Ch:= ReadKey;
        case Ch of
          '+': size:= sqrt ( 0.5 ) * size;
          '-': size:= sqrt ( 2 ) * size;
          chr ( 0 ):
            case ReadKey of
              chr ( $4B ): x0:= x0 - 0.1 * size;
              chr ( $4D ): x0:= x0 + 0.1 * size;
              chr ( $50 ): y0:= y0 - 0.1 * size;
              chr ( $48 ): y0:= y0 + 0.1 * size;
            end (* case *);
        end (* case *);
      until Ch = chr ( 27 );
      CloseGraph;
    end (* if *);
end.

